@@ -20,6 +20,7 @@ from .database_types import (
     TemporalType,
     UnknownColType,
     Text,
+    DbTime,
 )
 from data_diff.sql import DbPath, SqlOrStr, Compiler, Explain, Select, TableName
 
@@ -151,9 +152,10 @@ class Database(AbstractDatabase):
 
         elif issubclass(cls, Decimal):
             if numeric_scale is None:
-                raise ValueError(
-                    f"{self.name}: Unexpected numeric_scale is NULL, for column {'.'.join(table_path)}.{col_name} of type {type_repr}."
-                )
+                numeric_scale = 0  # Needed for Oracle.
+                # raise ValueError(
+                #     f"{self.name}: Unexpected numeric_scale is NULL, for column {'.'.join(table_path)}.{col_name} of type {type_repr}."
+                # )
             return cls(precision=numeric_scale)
 
         elif issubclass(cls, Float):
@@ -203,8 +205,7 @@ class Database(AbstractDatabase):
         fields = [self.normalize_uuid(c, String_UUID()) for c in text_columns]
         samples_by_row = self.query(Select(fields, TableName(table_path), limit=16), list)
         if not samples_by_row:
-            logger.warning(f"Table {table_path} is empty.")
-            return
+            raise ValueError(f"Table {table_path} is empty.")
 
         samples_by_col = list(zip(*samples_by_row))
 
@@ -242,6 +243,13 @@ class Database(AbstractDatabase):
 
         return f"LIMIT {limit}"
 
+    def concat(self, l: List[str]) -> str:
+        joined_exprs = ", ".join(l)
+        return f"concat({joined_exprs})"
+
+    def timestamp_value(self, t: DbTime) -> str:
+        return "'%s'" % t.isoformat()
+
     def normalize_uuid(self, value: str, coltype: ColType_UUID) -> str:
         if isinstance(coltype, String_UUID):
             return f"TRIM({value})"
