@@ -12,6 +12,8 @@ from .snowflake import Snowflake
 from .bigquery import BigQuery
 from .redshift import Redshift
 from .presto import Presto
+from .databricks import Databricks
+from .trino import Trino
 
 
 @dataclass
@@ -77,6 +79,12 @@ MATCH_URI_PATH = {
     ),
     "presto": MatchUriPath(Presto, ["catalog", "schema"], help_str="presto://<user>@<host>/<catalog>/<schema>"),
     "bigquery": MatchUriPath(BigQuery, ["dataset"], help_str="bigquery://<project>/<dataset>"),
+    "databricks": MatchUriPath(
+        Databricks,
+        ["catalog", "schema"],
+        help_str="databricks://:access_token@server_name/http_path",
+    ),
+    "trino": MatchUriPath(Trino, ["catalog", "schema"], help_str="trino://<user>@<host>/<catalog>/<schema>"),
 }
 
 
@@ -100,6 +108,8 @@ def connect_to_uri(db_uri: str, thread_count: Optional[int] = 1) -> Database:
     - bigquery
     - redshift
     - presto
+    - databricks
+    - trino
     """
 
     dsn = dsnparse.parse(db_uri)
@@ -113,23 +123,33 @@ def connect_to_uri(db_uri: str, thread_count: Optional[int] = 1) -> Database:
         raise NotImplementedError(f"Scheme {scheme} currently not supported")
 
     cls = matcher.database_cls
-    kw = matcher.match_path(dsn)
 
-    if scheme == "bigquery":
-        kw["project"] = dsn.host
-        return cls(**kw)
-
-    if scheme == "snowflake":
-        kw["account"] = dsn.host
-        assert not dsn.port
-        kw["user"] = dsn.user
-        kw["password"] = dsn.password
+    if scheme == "databricks":
+        assert not dsn.user
+        kw = {}
+        kw["access_token"] = dsn.password
+        kw["http_path"] = dsn.path
+        kw["server_hostname"] = dsn.host
+        kw.update(dsn.query)
     else:
-        kw["host"] = dsn.host
-        kw["port"] = dsn.port
-        kw["user"] = dsn.user
-        if dsn.password:
+        kw = matcher.match_path(dsn)
+
+        if scheme == "bigquery":
+            kw["project"] = dsn.host
+            return cls(**kw)
+
+        if scheme == "snowflake":
+            kw["account"] = dsn.host
+            assert not dsn.port
+            kw["user"] = dsn.user
             kw["password"] = dsn.password
+        else:
+            kw["host"] = dsn.host
+            kw["port"] = dsn.port
+            kw["user"] = dsn.user
+            if dsn.password:
+                kw["password"] = dsn.password
+
     kw = {k: v for k, v in kw.items() if v is not None}
 
     if issubclass(cls, ThreadedDatabase):
