@@ -29,14 +29,13 @@ class MySQL(ThreadedDatabase):
     }
     ROUNDS_ON_PREC_LOSS = True
 
-    def __init__(self, host, port, user, password, *, database, thread_count, **kw):
-        args = dict(host=host, port=port, database=database, user=user, password=password, **kw)
-        self._args = {k: v for k, v in args.items() if v is not None}
+    def __init__(self, *, thread_count, **kw):
+        self._args = kw
 
         super().__init__(thread_count=thread_count)
 
         # In MySQL schema and database are synonymous
-        self.default_schema = database
+        self.default_schema = kw["database"]
 
     def create_connection(self):
         mysql = import_mysql()
@@ -48,7 +47,7 @@ class MySQL(ThreadedDatabase):
             elif e.errno == mysql.errorcode.ER_BAD_DB_ERROR:
                 raise ConnectError("Database does not exist") from e
             else:
-                raise ConnectError(*e.args) from e
+                raise ConnectError(*e._args) from e
 
     def quote(self, s: str):
         return f"`{s}`"
