@@ -6,6 +6,8 @@ from datetime import datetime
 
 from runtype import dataclass
 
+from .utils import join_iter
+
 from .databases.database_types import AbstractDatabase, DbPath, DbKey, DbTime, ArithUUID
 
 
@@ -15,6 +17,8 @@ class Sql:
 
 SqlOrStr = Union[Sql, str]
 
+CONCAT_SEP = "|"
+
 
 @dataclass
 class Compiler:
@@ -121,8 +125,9 @@ class Checksum(Sql):
 
     def compile(self, c: Compiler):
         if len(self.exprs) > 1:
-            compiled_exprs = ", ".join(f"coalesce({c.compile(expr)}, '<null>')" for expr in self.exprs)
-            expr = f"concat({compiled_exprs})"
+            compiled_exprs = [f"coalesce({c.compile(expr)}, '<null>')" for expr in self.exprs]
+            separated = list(join_iter(f"'|'", compiled_exprs))
+            expr = c.database.concat(separated)
         else:
             # No need to coalesce - safe to assume that key cannot be null
             (expr,) = self.exprs
@@ -180,10 +185,9 @@ class Max(Sql):
 @dataclass
 class Time(Sql):
     time: datetime
-    column: Optional[SqlOrStr] = None
 
     def compile(self, c: Compiler):
-        return "'%s'" % self.time.isoformat()
+        return c.database.timestamp_value(self.time)
 
 
 @dataclass
