@@ -1,6 +1,7 @@
 import math
+from urllib.parse import urlparse
 
-from typing import Sequence, Optional, Tuple, Union, Dict, Any
+from typing import Union, Any
 from uuid import UUID
 
 
@@ -51,3 +52,28 @@ def number_to_human(n):
     )
 
     return "{:.0f}{}".format(n / 10 ** (3 * millidx), millnames[millidx])
+
+
+def _join_if_any(sym, args):
+    args = list(args)
+    if not args:
+        return ''
+    return sym.join(str(a) for a in args if a)
+
+def remove_password_from_url(url: str, replace_with: str="***") -> str:
+    parsed = urlparse(url)
+    account = parsed.username or ''
+    if parsed.password:
+        account += ':' + replace_with
+    host = _join_if_any(":", filter(None, [parsed.hostname, parsed.port]))
+    netloc = _join_if_any("@", filter(None, [account, host]))
+    replaced = parsed._replace(netloc=netloc)
+    return replaced.geturl()
+
+def join_iter(joiner: Any, iterable: iter) -> iter:
+    it = iter(iterable)
+    yield next(it)
+    for i in it:
+        yield joiner
+        yield i
+
